#include "i2c_eeprom.h"

/*typedef struct fileaddresses
{
  int startaddress;
  int endaddress;
};*/

fileaddresses fileA[25];
long currentMemaddress;
long startMemaddress;
long currentFileNbr = 1;
//================================================================
// EEPROM functions 
//================================================================
void i2c_eeprom_write_byte( int deviceaddress, unsigned int eeaddress, byte data ) {
  int rdata = data;
  int writeDelay=10;
  Wire.beginTransmission(deviceaddress);
  Wire.write((int)(eeaddress >> 8)); // MSB
  Wire.write((int)(eeaddress & 0xFF)); // LSB
  Wire.write(rdata);
  Wire.endTransmission();
  delay(writeDelay);
}

byte i2c_eeprom_read_byte( int deviceaddress, unsigned int eeaddress ) {
  byte rdata = 0xFF;
  int readDelay=5;
  Wire.beginTransmission(deviceaddress);
  Wire.write((int)(eeaddress >> 8)); // MSB
  Wire.write((int)(eeaddress & 0xFF)); // LSB
  Wire.endTransmission();
  Wire.requestFrom(deviceaddress,1);
  if (Wire.available()) rdata = Wire.read();
  return rdata;
}

void i2c_eeprom_erase_fileList()
{
  Serial.println(F("Start erasing files"));
  for (long i = 0; i < 101; i++)
  {
    i2c_eeprom_write_byte( 0x50, i, 0x00 );
  }
  currentFileNbr = 1;
  Serial.println(F("Done erasing file"));
}




//================================================================
// EEPROM Flight management functions 
//================================================================
int getFlightList()
{
  //get address where we are allowed to record
  //read the table to see what has been recorded so far

  int i = 0;
  //long 
  currentFileNbr = 1;
  byte startaddressMSB = B00000000;
  byte startaddressLSB = B00000000;
  byte endaddressMSB = B00000000;
  byte endaddressLSB = B00000000;
  boolean finish = false;

  while (i < 101 && finish == false)
  {
    int startaddress;
    int endaddress;
    startaddressMSB = i2c_eeprom_read_byte(0x50, i);
    i++;
    startaddressLSB = i2c_eeprom_read_byte(0x50, i);
    i++;
    startaddress = (startaddressMSB << 8) | startaddressLSB;

    endaddressMSB = i2c_eeprom_read_byte(0x50, i);
    i++;
    endaddressLSB = i2c_eeprom_read_byte(0x50, i);
    i++;
    endaddress = (endaddressMSB << 8) | endaddressLSB;

    fileA[currentFileNbr].startaddress = startaddress;
    fileA[currentFileNbr].endaddress = endaddress;

    currentFileNbr++;
    if (startaddress == 0) 
    {
      currentFileNbr = currentFileNbr -1;
      currentMemaddress =fileA[currentFileNbr-1].endaddress +1;
      finish = true;
    } 
  } 
  if (currentMemaddress == 1)
    currentMemaddress=101;

  startMemaddress = currentMemaddress; 
  //return the number of flight
  return (currentFileNbr-1);
}

boolean saveFlightA(int flightNbr, long startAdress, long endAdress)
{

  byte startaddressMSB = B00000000;
  byte startaddressLSB = B00000000;
  byte endaddressMSB = B00000000;
  byte endaddressLSB = B00000000;

  if (flightNbr < 26)
  {
    //save start address
    startaddressMSB = startMemaddress >> 8;
    i2c_eeprom_write_byte(0x50, (currentFileNbr*4)-4, startaddressMSB);

    startaddressLSB = startMemaddress;
    i2c_eeprom_write_byte(0x50, (currentFileNbr*4)-3, startaddressLSB); 
    endaddressMSB = (currentMemaddress -1) >> 8;
    i2c_eeprom_write_byte(0x50, (currentFileNbr*4)-2, endaddressMSB);
    endaddressLSB = (currentMemaddress -1);
    i2c_eeprom_write_byte(0x50, (currentFileNbr*4)-1, endaddressLSB);
    return true;
  }
  else
    return false;

}

void printFlightList()
{
  int startaddress;
  int endaddress;
  int flightNbr= 1;
  boolean exit=false;

  getFlightList();
  while(exit == false)
  {
    startaddress = fileA[flightNbr].startaddress;
    endaddress = fileA[flightNbr].endaddress;
    if (startaddress == 0) 
      exit = true;
    else
    {
      Serial.println("Flight Nbr: " + String(flightNbr) + " Start address: " + String(startaddress) + " End address: " + String(endaddress) + "\n");  
      flightNbr ++;
    }
  }
}

boolean isEepromFull()
{
  int nbrOfFlight = getFlightList();
  int endaddress = fileA[nbrOfFlight].endaddress;
  if (nbrOfFlight == 25)
    return true;
  if (endaddress == 65536)
    return true;
  return false;
}

void printFlight(int flightNbr)
{

  byte startaddressMSB = B00000000;
  byte startaddressLSB = B00000000;
  byte endaddressMSB = B00000000;
  byte endaddressLSB = B00000000;
  int startaddress;
  int endaddress;

  startaddressMSB = i2c_eeprom_read_byte(0x50, (flightNbr * 4)-4);

  startaddressLSB = i2c_eeprom_read_byte(0x50, (flightNbr * 4)-3);

  startaddress = (startaddressMSB << 8) | startaddressLSB;

  endaddressMSB = i2c_eeprom_read_byte(0x50, (flightNbr * 4)-2);

  endaddressLSB = i2c_eeprom_read_byte(0x50, (flightNbr * 4)-1);

  endaddress = (endaddressMSB << 8) | endaddressLSB;
  if (startaddress>100)
  {
    int i = startaddress;
    unsigned long currentTime=0;
    Serial.println("StartFlight;" );
    while(i < (endaddress +1))
    {

      byte currentTimeMSB = B00000000;
      byte currentTimeLSB = B00000000;
      byte currAltitudeMSB = B00000000;
      byte currAltitudeLSB = B00000000;

      currentTimeMSB = i2c_eeprom_read_byte(0x50, i);
      i++;
      currentTimeLSB = i2c_eeprom_read_byte(0x50, i);
      i++;

      currAltitudeMSB = i2c_eeprom_read_byte(0x50, i);
      i++;
      currAltitudeLSB = i2c_eeprom_read_byte(0x50, i);
      i++;
      currentTime = currentTime + ((currentTimeMSB << 8) | currentTimeLSB);
      //Serial.println(String(currentTime) + ","+ String((currAltitudeMSB << 8) | currAltitudeLSB) + "\n");
      Serial.println(String(currentTime) + ","+ String((currAltitudeMSB << 8) | currAltitudeLSB) );
      //Serial.println(String((currentTimeMSB << 8) | currentTimeLSB) + ","+ String((currAltitudeMSB << 8) | currAltitudeLSB) + "\n");
    }
    Serial.println("EndFlight;" );
  }
  else
    Serial.println(F("No such flight\n"));
}

void printFlightData(int flightNbr)
{

  byte startaddressMSB = B00000000;
  byte startaddressLSB = B00000000;
  byte endaddressMSB = B00000000;
  byte endaddressLSB = B00000000;
  int startaddress;
  int endaddress;

  startaddressMSB = i2c_eeprom_read_byte(0x50, (flightNbr * 4)-4);

  startaddressLSB = i2c_eeprom_read_byte(0x50, (flightNbr * 4)-3);

  startaddress = (startaddressMSB << 8) | startaddressLSB;

  endaddressMSB = i2c_eeprom_read_byte(0x50, (flightNbr * 4)-2);

  endaddressLSB = i2c_eeprom_read_byte(0x50, (flightNbr * 4)-1);

  endaddress = (endaddressMSB << 8) | endaddressLSB;
  if (startaddress>100)
  {
    int i = startaddress;
    unsigned long currentTime=0;
    //Serial.println("StartFlight;" );
    while(i < (endaddress +1))
    {

      byte currentTimeMSB = B00000000;
      byte currentTimeLSB = B00000000;
      byte currAltitudeMSB = B00000000;
      byte currAltitudeLSB = B00000000;

      currentTimeMSB = i2c_eeprom_read_byte(0x50, i);
      i++;
      currentTimeLSB = i2c_eeprom_read_byte(0x50, i);
      i++;

      currAltitudeMSB = i2c_eeprom_read_byte(0x50, i);
      i++;
      currAltitudeLSB = i2c_eeprom_read_byte(0x50, i);
      i++;
      currentTime = currentTime + ((currentTimeMSB << 8) | currentTimeLSB);
      //Serial.println(String(currentTime) + ","+ String((currAltitudeMSB << 8) | currAltitudeLSB) + "\n");
      Serial.println("$" + String("data,") +String(flightNbr) +","+String(currentTime) + ","+ String((currAltitudeMSB << 8) | currAltitudeLSB) +";");
      //Serial.println(String((currentTimeMSB << 8) | currentTimeLSB) + ","+ String((currAltitudeMSB << 8) | currAltitudeLSB) + "\n");
    }
    //Serial.println("EndFlight;" );
  }
  //else
  //Serial.println(F("No such flight\n"));
}
